<?php

use Illuminate\Database\Migrations\Migration;
use Illuminate\Database\Schema\Blueprint;

class EntrustSetupTables extends Migration {

  /**
   * Run the migrations.
   *
   * @return  void
   */
  public function up()
  {
    // Creates the roles table
    Schema::create('roles', function($table)
    {
      $table->increments('id')->unsigned();
      $table->string('name')->unique();
      $table->timestamp('created_at')->default(\DB::raw('CURRENT_TIMESTAMP'));
      $table->timestamp('updated_at')->nullable();
    });

    // Creates the assigned_roles (Many-to-Many relation) table
    Schema::create('assigned_roles', function($table)
    {
      $table->bigIncrements('id')->unsigned();
      $table->integer('user_id')->unsigned();
      $table->integer('role_id')->unsigned();
      $table->foreign('user_id')->references('id')->on('users')->onDelete('cascade'); // assumes a users table
      $table->foreign('role_id')->references('id')->on('roles')->onDelete('cascade');
    });

    // Creates the permissions table
    Schema::create('permissions', function($table)
    {
      $table->increments('id')->unsigned();
      $table->string('name');
      $table->string('display_name');
      $table->timestamp('created_at')->default(\DB::raw('CURRENT_TIMESTAMP'));
      $table->timestamp('updated_at')->nullable();
    });

    // Creates the permission_role (Many-to-Many relation) table
    Schema::create('permission_role', function($table)
    {
      $table->increments('id')->unsigned();
      $table->integer('permission_id')->unsigned();
      $table->integer('role_id')->unsigned();
      $table->foreign('permission_id')->references('id')->on('permissions')->onDelete('cascade'); // assumes a users table
      $table->foreign('role_id')->references('id')->on('roles')->onDelete('cascade');
    });
  }

  /**
   * Reverse the migrations.
   *
   * @return  void
   */
  public function down()
  {
    Schema::table('assigned_roles', function(Blueprint $table) {
      $table->dropForeign('assigned_roles_user_id_foreign');
      $table->dropForeign('assigned_roles_role_id_foreign');
    });

    Schema::table('permission_role', function(Blueprint $table) {
      $table->dropForeign('permission_role_permission_id_foreign');
      $table->dropForeign('permission_role_role_id_foreign');
    });

    Schema::drop('assigned_roles');
    Schema::drop('permission_role');
    Schema::drop('roles');
    Schema::drop('permissions');
  }

}
